package com.dsic.gj_erp.controller.zyjh;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.dsic.gj_erp.bean.ResultBean;
import com.dsic.gj_erp.bean.jhgk.DmYdjh;
import com.dsic.gj_erp.bean.zyjh.YcldwInfo;
import com.dsic.gj_erp.controller.zyjh.dto.DaoDuo;
import com.dsic.gj_erp.controller.zyjh.dto.ShangLiao;
import com.dsic.gj_erp.controller.zyjh.dto.SljhSearch;
import com.dsic.gj_erp.exception.CustomException;
import com.dsic.gj_erp.service.zyjh.YcldwInfoService;
import com.dsic.gj_erp.service.zyjh.YcldwService;
import com.dsic.gj_erp.service.zyjh.ZyjhService;
import com.dsic.gj_erp.ws.WsHandler;
import lombok.AllArgsConstructor;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;

import java.util.List;

@RestController
@RequestMapping("zyjh")
@AllArgsConstructor
public class ZyjhController {

    private final ZyjhService zyjhService;

    private final YcldwService ycldwService;

    private final YcldwInfoService ycldwInfoService;

    private final WsHandler wsHandler;

    /**
     * 获取上料计划,所有派工单数据整合到月度计划中统一管理,
     * 单独的派工单页面需要进行调整,从月度计划中获取对应工序数据
     */
    @PostMapping("sljh")
    public ResultBean<?> sljh(@RequestBody SljhSearch search){
        String date = DateUtil.date().toString("yyyy/MM/dd");
        if (StrUtil.isAllNotEmpty(search.getSljhrq())){
            date=search.getSljhrq();
        }
        List<DmYdjh> list = zyjhService.list(Wrappers.<DmYdjh>lambdaQuery()
                .le(DmYdjh::getSljhrq, date)
                .eq(DmYdjh::getZt, "2")
                .eq(StrUtil.isNotEmpty(search.getDcch()),DmYdjh::getDcCh, search.getDcch())
                .eq(StrUtil.isNotEmpty(search.getPl()),DmYdjh::getDcPl, search.getPl())
                .eq(StrUtil.isNotEmpty(search.getKw()),DmYdjh::getKw, search.getKw())
        );
        return new ResultBean<>(list);
    }

    /**
     * 获取预处理垛位信息
     */
    @PostMapping("ycldw")
    public ResultBean<?> ycldw(){
        return new ResultBean<>(ycldwService.ycldwList());
    }

    @PostMapping("sl")
    @Transactional(rollbackFor = Exception.class)
    public ResultBean<?> sl(@RequestBody ShangLiao sl) throws CustomException {
        if (!sl.checkCkwp()){
            //throw new CustomException(10009,"物品库存不存在,或库存信息错误");
        }

        //更新预处理垛位信息
        YcldwInfo info=ycldwInfoService.updateForSl(sl);

        //更新月度计划上料相关字段
        zyjhService.updateForSl(sl);

        //todo 更新库存信息

        wsHandler.sendSlComplete((JSONObject)JSONObject.toJSON(info));
        return new ResultBean<>();
    }

    /**
     * 倒垛
     */
    @PostMapping("daoduo")
    @Transactional(rollbackFor = Exception.class)
    public ResultBean<?> daoduo(@RequestBody DaoDuo dd){
        if (dd.getLastCeng()!=dd.getCeng()||dd.getLastDuo()!=dd.getDuo()){
            int moveY=dd.getLastCeng()-dd.getCeng();
            int y=moveY>0?1:-1;
            for (YcldwInfo info:dd.getMovedList()){
                ycldwInfoService.daoduo(info.getId(),y);
            }
            ycldwInfoService.daoduo(dd.getId(),dd.getCeng());
        }
        return new ResultBean<>();
    }

}
